# -*- coding: utf-8 -*-
'''
Created on 28/giu/2011

@author: simone cittadini
'''
import datetime

from django.db import models
from django_extensions.db.fields import AutoSlugField
from django.utils.translation import ugettext_lazy as _
from cms.models.fields import PlaceholderField

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel


MESI = {1 : 'Gennaio', 2 : 'Febbraio', 3 : 'Marzo', 4 : 'Aprile', 5 : 'Maggio',
        6 : 'Giugno', 7 : 'Luglio', 8 : 'Agosto', 9 : 'Settembre',
        10 : 'Ottobre', 11 : 'Novembre', 12 : 'Dicembre'}


class AreaPress(models.Model):
    """
    Un contenitore di notizie.
    """
    # Indica in quale posizione viene presentato all'iterno di una lista di
    #  aree press.
    ordinale_pagina = models.PositiveIntegerField(unique=True)
    numero_press_mostrate = models.PositiveIntegerField()
    numero_mesi_mostrati = models.PositiveIntegerField(default=1)
    nome = models.CharField(max_length=64)
    slug = AutoSlugField('slug', populate_from='nome', overwrite=True)
    # Attenzione, il render di questi placeholder non è testo ma un paragrafo
    #  html. Quindi eventuali parametri del <p> vanno aggiunti editando da cms
    #  e non nel template della app.
    placeholder_titolo = PlaceholderField(
                                'placeholder_titolo', 
                                related_name='placeholder_titolo_areapress'
                                )
    placeholder_sottotitolo = PlaceholderField(
                                'placeholder_sottotitolo', 
                                related_name='placeholder_sottotitolo_areapress'
                                )

    
    @staticmethod
    def get_aree(values=True, ordered=True):
        return AreaPress.objects.values_list('id', 'nome').order_by('ordinale_pagina')

    def get_presslist(self, anno=None, mese=None, num_mesi=None):
        """
        Torna una lista di press da anno-mese a num_mesi prima, il default è
        il mese corrente.
        """
        if not num_mesi:
            num_mesi = self.numero_mesi_mostrati
        data_fine = datetime.date.today()#.replace(day=1)
        if anno:
            data_fine = data_fine.replace(year=int(anno))
        if mese:
            data_fine = data_fine.replace(month=int(mese))
        data_inizio = data_fine - datetime.timedelta(weeks=(4 * num_mesi))

        if Press.objects.filter(area_press__exact=self,
                                date_p__range=(data_inizio, data_fine)).count()\
                                > self.numero_press_mostrate:
            qs = Press.objects.filter(area_press__exact=self,
                                      date_p__range=(data_inizio, data_fine)
                                      ).reverse()[0:self.numero_press_mostrate]
        else:
            qs = Press.objects.filter(area_press__exact=self,
                                      date_p__range=(data_inizio, data_fine)
                                      ).reverse()
        return qs

    def get_lastpress_month(self):
        """
        Torna il numerico del mese più recente contenente una press.
        ( o il corrente se l'area è vuota )
        """
        qs = Press.objects.filter(area_press__exact=self).reverse()
        if qs.count() > 0:
            return qs[0:1][0].date_p.month
        else:
            return datetime.date.today().month
        
    def get_lastpress_year(self):
        """
        Torna il numerico del mese più recente contenente una press.
        ( o il corrente se l'area è vuota )
        """
        qs = Press.objects.filter(area_press__exact=self).reverse()
        if qs.count() > 0:
            return qs[0:1][0].date_p.year
        else:
            return datetime.date.today().year
        
    def __unicode__(self):
        return u"%s - %s" % (self.ordinale_pagina, self.nome)
    
    class Meta:
        verbose_name_plural = ('AreePress')
        

class Source(DateModel, UserModel, OrderedModel, StatusModel):
    name = models.CharField(unique = True, max_length = 100, db_index = True,
                            verbose_name = _('Name'))
    slug = models.SlugField(max_length = 100, db_index = True,
                            verbose_name = _('Slug'))

    class Meta:
        verbose_name = _("Source")
        verbose_name_plural = _("Sources")
        ordering = ['name',]

    def __unicode__(self):
        return u"%s" % self.name
 
    
class Press(DateModel, UserModel, OrderedModel, StatusModel):
    date_p = models.DateField(db_index = True,
                              verbose_name = _("Date"))
    title = models.CharField(max_length = 300, db_index = True,
                             verbose_name = _("Testata"))
    slug = models.SlugField(
        max_length = 100,
        db_index = True,
        verbose_name = _('Slug'),
    )
    source = models.ForeignKey(
        Source,
        null = True, blank = True,
        db_index = True,
        related_name = "presses",
        verbose_name = _("Source"),
    )
    file = models.FileField (
        null = True, blank = True,           
        upload_to = "press",
        verbose_name = _("File"),
    )
    abstract = models.TextField (
        null = True, blank = True,
        verbose_name = _("Titolo"),
    )
    keywords = models.CharField(
        null = True, blank = True,
        max_length = 200,
        db_index = True,
        verbose_name = _("Keywords"),
    )
    area_press = models.ForeignKey(
        AreaPress, 
        related_name = "press_area")


    class Meta:
        verbose_name = _("Press")
        verbose_name_plural = _("Press")
        ordering = ['date_p', 'title']

    def __unicode__(self):
        return u"%s" % self.title
    
    @staticmethod
    def get_meseanno(values=True):
        output = []
        for p in Press.objects.values_list('date_p').order_by('date_p'):
            if (datetime.datetime(p[0].year, p[0].month, 1), '%s %s' % (MESI[p[0].month], p[0].year)) not in output:
                output.append(
                          (datetime.datetime(p[0].year, p[0].month, 1),
                           '%s %s' % (MESI[p[0].month], p[0].year))
                          )
        output.reverse()
        return output
    
    @staticmethod
    def get_testate(values=True, from_title=True):
        output = []
        for p in Press.objects.values_list('title', 'title').order_by('title'):
            if p not in output:
                output.append(p)
        return output
    
    def get_short_abstract(self, limit=60):
        return "%s..." % self.abstract[0:limit]
